/*
 *   ORXONOX - the hottest 3D action shooter ever to exist
 *                    > www.orxonox.net <
 *
 *
 *   License notice:
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License
 *   as published by the Free Software Foundation; either version 2
 *   of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 *   Author:
 *      Yuning Chai
 *      Felix Schulthess
 *   Co-authors:
 *      Reto Grieder
 *      Wolfgang Roenninger
 *
 */

#include "HUDWeapon.h"

#include <OgreOverlayManager.h>
#include <OgrePanelOverlayElement.h>

#include "core/CoreIncludes.h"
#include "core/XMLPort.h"
#include "tools/TextureGenerator.h"
#include "util/Convert.h"
#include "core/class/Super.h"

namespace orxonox
{
    RegisterClass(HUDWeapon);

    HUDWeapon::HUDWeapon(Context* context) : OrxonoxOverlay(context)
    {
        RegisterObject(HUDWeapon);

        overlayElement_ = static_cast<Ogre::PanelOverlayElement* >(Ogre::OverlayManager::getSingleton().createOverlayElement("Panel", "HUDWeapon" + getUniqueNumberString()));
        overlayElement_->setMaterialName("Orxonox/WSHUD_Weapon");
        overlayElement_->setPosition(0.0f,0.0f);
        overlayElement_->setDimensions(1.0f,1.0f);
        this->background_->addChild(overlayElement_);
    }

    HUDWeapon::~HUDWeapon()
    {
        if (this->isInitialized())
        {
            destroyHUDChilds();
            //this->overlayElement_->destroy();            
        }
    }

    void HUDWeapon::XMLPort(Element& xmlelement, XMLPort::Mode mode)
    {
        SUPER(HUDWeapon, XMLPort, xmlelement, mode);

        /*XMLPortParam(HUDWeapons, "sensitivity", setRadarSensitivity, getRadarSensitivity, xmlelement, mode);
        XMLPortParam(HUDWeapons, "halfDotSizeDistance", setHalfDotSizeDistance, getHalfDotSizeDistance, xmlelement, mode);*/
    }

    void HUDWeapon::tick(float dt)
    {
        SUPER(HUDWeapon, tick, dt);

        if (this->owner_)
        {

        }
    }    

    void HUDWeapon::positionChanged()
    {
        OrxonoxOverlay::positionChanged();

        positionHUDChilds();
    }     

    void HUDWeapon::changedOwner()
    {
        SUPER(HUDWeapon, changedOwner);

        this->owner_ = orxonox_cast<Pawn*>(this->getOwner());

        updateWeaponModeList();
    }

    void HUDWeapon::changedOverlayGroup()
    {
        SUPER(HUDWeapon, changedOverlayGroup);

        //this->getOverlayGroup()->addElement(this->textOverlay_.get());
    }   

    void HUDWeapon::changedVisibility()
    {
        SUPER(HUDWeapon, changedVisibility);

        //this->textOverlay_->setVisible(this->isVisible());
    }

    void HUDWeapon::changedName()
    {
        SUPER(HUDWeapon, changedName);

        //this->textOverlay_->setName(this->getName() + "text");
    }

    void HUDWeapon::setWeapon(Weapon* weapon)
    {
        weapon_ = weapon;

        if (!weapon_)
        {
            return;
        }

        updateWeaponModeList();
    }

    void HUDWeapon::updateWeaponModeList()
    {
        if (owner_ == NULL || weapon_ == NULL)
        {
            return;
        }
            
        destroyHUDChilds();

        weaponModes_ = weapon_->getAllWeaponmodes();

        createHUDChilds();
        positionHUDChilds();        
    }  

    void HUDWeapon::createHUDChilds()
    {
        for (std::multimap<unsigned int, WeaponMode*>::iterator it = weaponModes_->begin(); it != weaponModes_->end(); ++it)
        {
            HUDWeaponMode* hudWeaponMode = new HUDWeaponMode(this->getContext());
            hudWeaponMode->setOwner(owner_);
            hudWeaponMode->setWeaponMode(it->second);
            hudWeaponModes_.push_back(hudWeaponMode);
        }    
    }      

    void HUDWeapon::positionHUDChilds()
    {
        int positionIndex = 0;

        for (std::vector<WeakPtr<HUDWeaponMode> >::iterator it = hudWeaponModes_.begin(); it != hudWeaponModes_.end(); ++it)
        {
            (*it)->setOverlayGroup(this->getOverlayGroup());            
            (*it)->setAspectCorrection(this->getAspectCorrection());
            (*it)->setPosition(this->getPosition() + Vector2(0.0f,0.025f*positionIndex));
            (*it)->setSize(Vector2(0.15f,0.025f));
            (*it)->setPickPoint(Vector2(0.0f,0.0f));

            ++ positionIndex;        
        }

        overlayElement_->setDimensions(1.0f,0.25f*hudWeaponModes_.size());
    }  

    void HUDWeapon::destroyHUDChilds()
    {
        for (std::vector<WeakPtr<HUDWeaponMode> >::iterator it = hudWeaponModes_.begin(); it != hudWeaponModes_.end(); ++it)
        {
            (*it)->destroy();
        } 

        hudWeaponModes_.clear();
    }
}
