/*
 *   ORXONOX - the hottest 3D action shooter ever to exist
 *                    > www.orxonox.net <
 *
 *
 *   License notice:
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License
 *   as published by the Free Software Foundation; either version 2
 *   of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 *   Author:
 *      Yuning Chai
 *      Felix Schulthess
 *   Co-authors:
 *      Reto Grieder
 *      Wolfgang Roenninger
 *
 */

#include "HUDWeaponMode.h"

#include <OgreOverlayManager.h>
#include <OgrePanelOverlayElement.h>

#include "core/CoreIncludes.h"
#include "core/XMLPort.h"
#include "tools/TextureGenerator.h"
#include "weaponsystem/WeaponMode.h"
#include "weaponsystem/Munition.h"
#include "util/Convert.h"

namespace orxonox
{
    RegisterClass(HUDWeaponMode);

    HUDWeaponMode::HUDWeaponMode(Context* context) : OrxonoxOverlay(context)
    {
        RegisterObject(HUDWeaponMode);

        overlayElementIcon_ = static_cast<Ogre::PanelOverlayElement* >(Ogre::OverlayManager::getSingleton().createOverlayElement("Panel", "HUDWeaponMode" + getUniqueNumberString()));
        overlayElementIcon_->setPosition(0.0f,0.0f);
        overlayElementIcon_->setDimensions(1.0f,1.0f);
        this->background_->addChild(overlayElementIcon_);

        overlayElementReplenish_ = static_cast<Ogre::PanelOverlayElement* >(Ogre::OverlayManager::getSingleton().createOverlayElement("Panel", "HUDWeaponMode" + getUniqueNumberString()));
        overlayElementReplenish_->setPosition(0.0f,0.0f);
        overlayElementReplenish_->setDimensions(1.0f,1.0f);
        this->background_->addChild(overlayElementReplenish_);

        overlayElementMunition_ = static_cast<Ogre::PanelOverlayElement* >(Ogre::OverlayManager::getSingleton().createOverlayElement("Panel", "HUDWeaponMode" + getUniqueNumberString()));
        overlayElementMunition_->setPosition(0.0f,0.0f);
        overlayElementMunition_->setDimensions(1.0f,1.0f);
        this->background_->addChild(overlayElementMunition_);

        overlayElementState_ = static_cast<Ogre::PanelOverlayElement* >(Ogre::OverlayManager::getSingleton().createOverlayElement("Panel", "HUDWeaponMode" + getUniqueNumberString()));
        overlayElementState_->setPosition(0.0f,0.0f);
        overlayElementState_->setDimensions(1.0f,1.0f);
        this->background_->addChild(overlayElementState_);

        overlayElementIcon_->show();
        overlayElementReplenish_->show();
        overlayElementMunition_->show();
        overlayElementState_->show();

        // Create two text overlays
        this->textOverlayLeft_ = new OverlayText(this->getContext());
        assert(this->textOverlayLeft_.get());
        this->textOverlayLeft_->setCaption("???");
        textOverlayLeft_->setPickPoint(Vector2(0.0f,0.0f));
        textOverlayLeft_->setVisible(true);
        textOverlayLeft_->setAlignment(OverlayText::Left);
        textOverlayLeft_->setTextSize(0.02f);
        textOverlayLeft_->setColour(ColourValue(0.21,0.70,0.21,1.0));
        textOverlayLeft_->setPosition(getPosition());
        textOverlayLeft_->setAspectCorrection(this->getAspectCorrection());
        
        this->textOverlayRight_ = new OverlayText(this->getContext());
        assert(this->textOverlayLeft_.get());
        this->textOverlayRight_->setCaption("???");
        textOverlayRight_->setPickPoint(Vector2(0.0f,0.0f));
        textOverlayRight_->setVisible(true);
        textOverlayRight_->setAlignment(OverlayText::Left);
        textOverlayRight_->setTextSize(0.02f);
        textOverlayRight_->setColour(ColourValue(0.21,0.70,0.21,1.0));
        textOverlayRight_->setPosition(getPosition());
        textOverlayRight_->setAspectCorrection(this->getAspectCorrection());        

        materialNameState_ = "Orxonox/WSHUD_Reloading";

        overlayElementReplenish_->setMaterialName("Orxonox/WSHUD_Replenish");
    }

    HUDWeaponMode::~HUDWeaponMode()
    {
        if (this->isInitialized())
        {
            //this->textOverlay_->destroy();            
        }
    }

    void HUDWeaponMode::tick(float dt)
    {
        SUPER(HUDWeaponMode, tick, dt);

        if (this->owner_ && this->weaponMode_)
        {
            std::string lastMaterialNameState = materialNameState_;

            if (weaponMode_->getReloading())
            {
                materialNameState_ = "Orxonox/WSHUD_Reloading";
            }
            else
            {
                materialNameState_ = "Orxonox/WSHUD_Ready";
            }            

            Munition* munition = this->weaponMode_->getMunition();

            if (munition != NULL)
            {
                bool useSeparateMagazines = munition->getUseSeparateMagazines();
                bool stackMunition = munition->getStackMunition();

                if (!useSeparateMagazines && !stackMunition)
                {
                    this->textOverlayLeft_->setCaption(multi_cast<std::string>(munition->getNumMunitionInCurrentMagazine(weaponMode_)));
                    this->textOverlayRight_->setCaption(multi_cast<std::string>(munition->getNumMagazines()));
                }
                else if (!useSeparateMagazines && stackMunition)
                {
                    this->textOverlayLeft_->setCaption(multi_cast<std::string>(munition->getNumMunition(weaponMode_)));
                    this->textOverlayRight_->setCaption("");
                }
                else if (useSeparateMagazines && !stackMunition)
                {
                    this->textOverlayLeft_->setCaption(multi_cast<std::string>(munition->getNumMunitionInCurrentMagazine(weaponMode_)));
                    this->textOverlayRight_->setCaption(multi_cast<std::string>(munition->getNumMagazines()));
                }

                if (munition->getNumMunition(weaponMode_) == 0)
                {
                    materialNameState_ = "Orxonox/WSHUD_Empty";
                }
            }

            if (munition->isA(Class(ReplenishingMunition)))
            {
                ReplenishingMunition* replenishingMunition = dynamic_cast<ReplenishingMunition*>(munition);

                if (replenishingMunition->canAddMunition(1))
                {
                    float progress = 1.0f - replenishingMunition->getProgress();
                    overlayElementReplenish_->setDimensions(1.0f,progress);                    
                }
                else
                {
                    overlayElementReplenish_->setDimensions(1.0f,1.0f);
                }
            }

            if (materialNameState_ != lastMaterialNameState)
            {
                overlayElementState_->setMaterialName(materialNameState_);
            }
        }
    }    

    void HUDWeaponMode::positionChanged()
    {
        OrxonoxOverlay::positionChanged();

        positionHUDChilds();
    }    

    void HUDWeaponMode::changedOwner()
    {
        SUPER(HUDWeaponMode, changedOwner);

        this->owner_ = orxonox_cast<Pawn*>(this->getOwner());

        //updateWeaponsList();
    }

    void HUDWeaponMode::changedOverlayGroup()
    {
        SUPER(HUDWeaponMode, changedOverlayGroup);

        this->getOverlayGroup()->addElement(this->textOverlayLeft_.get());
        this->getOverlayGroup()->addElement(this->textOverlayRight_.get());
    }   

    void HUDWeaponMode::changedVisibility()
    {
        SUPER(HUDWeaponMode, changedVisibility);

        //this->textOverlay_->setVisible(this->isVisible());
    }

    void HUDWeaponMode::changedName()
    {
        SUPER(HUDWeaponMode, changedName);

        //this->textOverlay_->setName(this->getName() + "text");
    }  

    void HUDWeaponMode::setWeaponMode(WeaponMode* weaponMode)
    {
        weaponMode_ = weaponMode;

        if (!weaponMode_)
        {
            return;
        }

        std::string materialName = weaponMode_->getHUDImageString();

        overlayElementIcon_->setMaterialName(materialName);

        Munition* munition = this->weaponMode_->getMunition();

        if (munition != NULL)
        {
            bool useSeparateMagazines = munition->getUseSeparateMagazines();
            bool stackMunition = munition->getStackMunition();

            if (!useSeparateMagazines && !stackMunition)
            {
                overlayElementMunition_->setMaterialName("Orxonox/WSHUD_MunitionShare");
            }
            else if (!useSeparateMagazines && stackMunition)
            {
                overlayElementMunition_->setMaterialName("Orxonox/WSHUD_MunitionStack");
            }
            else if (useSeparateMagazines && !stackMunition)
            {
                overlayElementMunition_->setMaterialName("Orxonox/WSHUD_MunitionSeparate");
            }
        }      
    }

    void HUDWeaponMode::positionHUDChilds()
    {
        textOverlayLeft_->setPosition(getPosition() + Vector2(0.06f,0.0f));
        textOverlayRight_->setPosition(getPosition() + Vector2(0.105f,0.0f));
    }  
}
