/*
 *   ORXONOX - the hottest 3D action shooter ever to exist
 *                    > www.orxonox.net <
 *
 *
 *   License notice:
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License
 *   as published by the Free Software Foundation; either version 2
 *   of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 *   Author:
 *      Yuning Chai
 *      Felix Schulthess
 *   Co-authors:
 *      Reto Grieder
 *      Wolfgang Roenninger
 *
 */

#include "HUDWeaponSystem.h"

#include <OgreOverlayManager.h>
#include <OgrePanelOverlayElement.h>

#include "core/CoreIncludes.h"
#include "core/XMLPort.h"
#include "tools/TextureGenerator.h"
#include "weaponsystem/WeaponSystem.h"
#include "weaponsystem/WeaponPack.h"
#include "weaponsystem/Weapon.h"
#include "util/Convert.h"
#include "core/class/Super.h"

namespace orxonox
{
    RegisterClass(HUDWeaponSystem);

    HUDWeaponSystem::HUDWeaponSystem(Context* context) : OrxonoxOverlay(context)
    {
        RegisterObject(HUDWeaponSystem);

        weapons_.clear();
        hudWeapons_.clear();
    }

    HUDWeaponSystem::~HUDWeaponSystem()
    {
        if (this->isInitialized())
        {
            destroyHUDChilds();
        }
    }

    void HUDWeaponSystem::XMLPort(Element& xmlelement, XMLPort::Mode mode)
    {
        SUPER(HUDWeaponSystem, XMLPort, xmlelement, mode);

        /*XMLPortParam(HUDWeaponSystem, "sensitivity", setRadarSensitivity, getRadarSensitivity, xmlelement, mode);
        XMLPortParam(HUDWeaponSystem, "halfDotSizeDistance", setHalfDotSizeDistance, getHalfDotSizeDistance, xmlelement, mode);*/
    }

    void HUDWeaponSystem::tick(float dt)
    {
        SUPER(HUDWeaponSystem, tick, dt);

        if (this->owner_)
        {

        }
    }  

    void HUDWeaponSystem::positionChanged()
    {
        OrxonoxOverlay::positionChanged();

        positionHUDChilds();
    }  

    void HUDWeaponSystem::changedOwner()
    {
        SUPER(HUDWeaponSystem, changedOwner);

        this->owner_ = orxonox_cast<Pawn*>(this->getOwner());

        updateWeaponList();
    }

    void HUDWeaponSystem::changedOverlayGroup()
    {
        SUPER(HUDWeaponSystem, changedOverlayGroup);

        //this->getOverlayGroup()->addElement(this->textOverlay_.get());
    }   

    void HUDWeaponSystem::changedVisibility()
    {
        SUPER(HUDWeaponSystem, changedVisibility);

        //this->textOverlay_->setVisible(this->isVisible());
    }

    void HUDWeaponSystem::changedName()
    {
        SUPER(HUDWeaponSystem, changedName);

        //this->textOverlay_->setName(this->getName() + "text");
    }  

    void HUDWeaponSystem::updateWeaponList()
    {
        if (owner_ == NULL)
        {
            return;
        }

        weapons_.clear();

        destroyHUDChilds();

        std::vector<WeaponPack*>* weaponPacks = owner_->getAllWeaponPacks();

        for (std::vector<WeaponPack*>::const_iterator itPacks = weaponPacks->begin(); itPacks != weaponPacks->end(); ++itPacks)
        {
            std::vector<Weapon*>* weapons = (*itPacks)->getAllWeapons();

            for (std::vector<Weapon*>::const_iterator itWeapons = weapons->begin(); itWeapons != weapons->end(); ++itWeapons)
            {
                this->weapons_.push_back(*itWeapons);
            }
        }

        createHUDChilds();
        positionHUDChilds();        
    }

    void HUDWeaponSystem::createHUDChilds()
    {
        for (std::vector<WeakPtr<Weapon> >::iterator it = weapons_.begin(); it != weapons_.end(); ++it)
        {
            HUDWeapon* hudWeapon = new HUDWeapon(this->getContext());            
            hudWeapon->setOwner(owner_);
            hudWeapon->setWeapon(*it);
            hudWeapons_.push_back(hudWeapon);
        }       
    }      

    void HUDWeaponSystem::positionHUDChilds()
    {
        int positionIndex = 0;

        for (std::vector<WeakPtr<HUDWeapon> >::iterator it = hudWeapons_.begin(); it != hudWeapons_.end(); ++it)
        {
            (*it)->setOverlayGroup(this->getOverlayGroup());            
            (*it)->setAspectCorrection(this->getAspectCorrection());
            (*it)->setPosition(this->getPosition() + Vector2(0.15f*positionIndex,0.0f));            
            (*it)->setSize(Vector2(0.15f,0.10f));
            (*it)->setPickPoint(Vector2(0.0f,0.0f));

            ++ positionIndex;
        }        
    }  

    void HUDWeaponSystem::destroyHUDChilds()
    {
        for (std::vector<WeakPtr<HUDWeapon> >::iterator it = hudWeapons_.begin(); it != hudWeapons_.end(); ++it)
        {
            (*it)->destroy();
        } 

        hudWeapons_.clear();     
    }     
}
